window.Gleam.initSearch([{"doc":"tardis","title":"tardis","content":"# Tardis\n\n&lt;!-- [![Package Version](https://img.shields.io/hexpm/v/tardis)](https://hex.pm/packages/tardis)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/tardis/) --&gt;\n\n&gt; While fully working, and usable right away, Tardis is still in development to\n&gt; find the perfect API and to see how to integrate it in Lustre &amp; the Lustre\n&gt; devtools. Maybe in few months, you&#39;ll be able to enjoy the time-traveller\n&gt; debugger without any install needed! Right now, Tardis uses internals access\n&gt; from Lustre, otherwise it could not work. Be careful when updating Lustre,\n&gt; otherwise Tardis could end up not working! You can expect quick releases\n&gt; though at each Lustre release!\n&gt;\n&gt; If you think to anything, feedbacks are greatly appreciated!\n\nEvery good frontend framework deserve to have a good debugger. Tardis tries to\nfill this gap with [Lustre](https://hexdocs.pm/lustre). Because of the immutable\nnature and the management of side-effects of lustre, it&#39;s possible to implement\na debugger able to register everything that happened in the app and that can\nrewind the time in order to display the state of your app, at any point in time!\nTardis is a time-traveller debugger, made to interact with multiple lustre\napplications and components on one page, with the simplest setup possible yet!\n\n## Demo\n\nhttps://github.com/ghivert/tardis/assets/7314118/748ed6bd-b593-47c8-bde9-647c67e0e25e\n\n## Quickstart Guide\n\nFirst, add tardis to your gleam project.\n\n```sh\ngleam add tardis\n```\n\nThen, setup the package.\n\n```gleam\nimport gleam/int\nimport lustre\nimport lustre/element/html\nimport lustre/event\nimport tardis\n\npub fn main() {\n  let assert Ok(main) = tardis.single(&quot;main&quot;)\n\n  lustre.application(init, update, view)\n  |&gt; tardis.wrap(with: main)\n  |&gt; lustre.start(&quot;#app&quot;, Nil)\n  |&gt; tardis.activate(with: main)\n}\n\nfn init(_) {\n  0\n}\n\nfn update(model, msg) {\n  case msg {\n    Incr -&gt; model + 1\n    Decr -&gt; model - 1\n  }\n}\n\nfn view(model) {\n  let count = int.to_string(model)\n  html.div([], [\n    html.button([event.on_click(Incr)], [html.text(&quot; + &quot;)]),\n    html.p([], [html.text(count)]),\n    html.button([event.on_click(Decr)], [html.text(&quot; - &quot;)])\n  ])\n}\n```\n\nYou&#39;re good to go!\n\n## Multiple apps setup\n\nWhile it&#39;s easy to setup a single application with tardis, it can also be used\nto debug multiple applications in the same page. Tardis exposes two additional\nfunctions: [`setup`](https://hexdocs.pm/tardis/tardis.html#setup) and\n[`application`](https://hexdocs.pm/tardis/tardis.html#application). The first\none initialize the debugger, while the second one allows to setup an application\non the debugger!\n\nIn case you&#39;re developping a independant package, you can even send the tardis\nor the debugger instance directly to your application, and it will nicely\nintegrate in it!\n\n```gleam\nimport gleam/int\nimport lustre\nimport lustre/element/html\nimport lustre/event\nimport tardis\n\npub fn main() {\n  let assert Ok(instance) = tardis.setup()\n  let main = tardis.application(instance, &quot;main&quot;)\n  let mod = tardis.application(instance, &quot;module&quot;)\n\n  lustre.application(init_1, update_1, view_1)\n  |&gt; tardis.wrap(with: main)\n  |&gt; lustre.start(&quot;#app&quot;, Nil)\n  |&gt; tardis.activate(with: main)\n\n  lustre.application(init_2, update_2, view_2)\n  |&gt; tardis.wrap(with: mod)\n  |&gt; lustre.start(&quot;#mod&quot;, Nil)\n  |&gt; tardis.activate(with: mod)\n}\n```\n\n## Style collision\n\nNo worry about the debugger going into your application! Tardis uses the Shadow\nDOM, meaning no style nor behavior will leak out of the debugger and ending in\nyour application. Tardis will just come on top, watch the application, and can\nrollback in time. Nothing more!\n","url":"index.html"},{"doc":"tardis/error","title":"Error","content":"pub type Error {\n  SketchError(sketch.SketchError)\n  LustreError(lustre.Error)\n}\n\nSketchError(sketch.SketchError)\n\n\nLustreError(lustre.Error)\n\n\nSynonyms:\ntardis/error.Error\ntardis/error Error","url":"tardis/error.html#Error"},{"doc":"tardis/error","title":"tardis/error","content":" Manages error for Tardis.\n Used only during initialization, to identify what is failing.","url":"tardis/error.html"},{"doc":"tardis","title":"Instance","content":"pub opaque type Instance\n Represents the instance for an application. It should be used within one and\n only one application. You can get the instance by using [`application`](#application)\n or [`single`](#single).\n\nSynonyms:\ntardis.Instance\ntardis Instance","url":"tardis.html#Instance"},{"doc":"tardis","title":"Tardis","content":"pub opaque type Tardis\n Represents the running Tardis. Should be used with lustre applications\n only. One tardis is enough for multiple lustre applications running on the\n same page, with their own update loop.\n\nSynonyms:\ntardis.Tardis\ntardis Tardis","url":"tardis.html#Tardis"},{"doc":"tardis","title":"activate","content":"pub fn activate(\n  result: Result(fn(Action(a, b)) -&gt; Nil, c),\n  with instance: Instance,\n) -&gt; Result(fn(Action(a, b)) -&gt; Nil, c)\n Activate the debugger of the application. `activate` should be run to let\n the debugger being able to rewind time. It should be executed on the exact\n same instance that has been wrapped.\n \n fn main() {\n   let assert Ok(instance) = tardis.single(&quot;main&quot;)\n   lustre.application(init, update, view)\n   |&gt; tardis.wrap(with: instance)\n   |&gt; lustre.start()\n   |&gt; tardis.activate(with: instance)\n }\n \nSynonyms:\ntardis.activate\ntardis activate","url":"tardis.html#activate"},{"doc":"tardis","title":"application","content":"pub fn application(instance: Tardis, name: String) -&gt; Instance\n Creates the application debugger from the tardis. Should be run once,\n at the start of the application. It can be skipped when using [`single`](#single).\nSynonyms:\ntardis.application\ntardis application","url":"tardis.html#application"},{"doc":"tardis","title":"setup","content":"pub fn setup() -&gt; Result(Tardis, Error)\n Creates the tardis. Should be run once, at the start of the application.\n It can be skipped when using [`single`](#single).\nSynonyms:\ntardis.setup\ntardis setup","url":"tardis.html#setup"},{"doc":"tardis","title":"single","content":"pub fn single(name: String) -&gt; Result(Instance, Error)\n Directly creates a tardis instance for a single application.\n Replaces `setup` and `application` in a single application context.\nSynonyms:\ntardis.single\ntardis single","url":"tardis.html#single"},{"doc":"tardis","title":"wrap","content":"pub fn wrap(\n  application: App(a, b, c),\n  with instance: Instance,\n) -&gt; App(a, b, c)\n Wrap a lustre application with the debugger. The debugger will never interfere\n with your application by itself. You can directly chain your application with\n this function.\n \n fn main() {\n   let assert Ok(instance) = tardis.single(&quot;main&quot;)\n   lustre.application(init, update, view)\n   |&gt; tardis.wrap(with: instance)\n }\n \nSynonyms:\ntardis.wrap\ntardis wrap","url":"tardis.html#wrap"},{"doc":"tardis","title":"tardis","content":" Tardis is in charge of being your good debugger friend, able to navigate\n accross time and space! Just instanciate it, and use it in your application.\n\n ```gleam\n import gleam/int\n import lustre\n import lustre/element/html\n import lustre/event\n import tardis\n\n pub fn main() {\n   let assert Ok(main) = tardis.single(&quot;main&quot;)\n\n   lustre.application(init, update, view)\n   |&gt; tardis.wrap(with: main)\n   |&gt; lustre.start(&quot;#app&quot;, Nil)\n   |&gt; tardis.activate(with: main)\n }\n\n fn init(_) {\n   0\n }\n\n fn update(model, msg) {\n   case msg {\n     Incr -&gt; model + 1\n     Decr -&gt; model - 1\n   }\n }\n\n fn view(model) {\n   let count = int.to_string(model)\n   html.div([], [\n     html.button([event.on_click(Incr)], [html.text(&quot; + &quot;)]),\n     html.p([], [html.text(count)]),\n     html.button([event.on_click(Decr)], [html.text(&quot; - &quot;)])\n   ])\n }\n ```","url":"tardis.html"}]);